package sun.invoke.util;

import java.lang.invoke.MethodType;
import java.security.AccessController;
import java.security.PrivilegedAction;
import sun.reflect.Reflection;
import java.lang.reflect.Modifier;

public class VerifyAccess
{
    private static final int PACKAGE_ONLY = 0;
    private static final int PACKAGE_ALLOWED = 8;
    private static final int PROTECTED_OR_PACKAGE_ALLOWED = 12;
    private static final int ALL_ACCESS_MODES = 7;
    private static final boolean ALLOW_NESTMATE_ACCESS = false;
    
    private VerifyAccess() {
    }
    
    public static boolean isMemberAccessible(Class<?> clazz, Class<?> clazz2, int mod, Class<?> clazz3, int n) {
        if (n == 0) {
            return false;
        }
        assert (n & 0x1) != 0x0 && (n & 0xFFFFFFF0) == 0x0;
        if (!isClassAccessible(clazz, clazz3, n)) {
            return false;
        }
        if (clazz2 == clazz3 && (n & 0x2) != 0x0) {
            return true;
        }
        switch (mod & 0x7) {
            case 1: {
                return true;
            }
            case 4: {
                assert !clazz2.isInterface();
                return ((n & 0xC) != 0x0 && isSamePackage(clazz2, clazz3)) || ((n & 0x4) != 0x0 && ((mod & 0x8) == 0x0 || isRelatedClass(clazz, clazz3)) && ((n & 0x4) != 0x0 && isSubClass(clazz3, clazz2)));
            }
            case 0: {
                assert !clazz2.isInterface();
                return (n & 0x8) != 0x0 && isSamePackage(clazz2, clazz3);
            }
            case 2: {
                return false;
            }
            default: {
                throw new IllegalArgumentException("bad modifiers: " + Modifier.toString(mod));
            }
        }
    }
    
    static boolean isRelatedClass(Class<?> clazz, Class<?> clazz2) {
        return clazz == clazz2 || isSubClass(clazz, clazz2) || isSubClass(clazz2, clazz);
    }
    
    static boolean isSubClass(Class<?> clazz, Class<?> clazz2) {
        return clazz2.isAssignableFrom(clazz) && !clazz.isInterface();
    }
    
    static int getClassModifiers(Class<?> clazz) {
        if (clazz.isArray() || clazz.isPrimitive()) {
            return clazz.getModifiers();
        }
        return Reflection.getClassAccessFlags(clazz);
    }
    
    public static boolean isClassAccessible(Class<?> clazz, Class<?> clazz2, int n) {
        if (n == 0) {
            return false;
        }
        assert (n & 0x1) != 0x0 && (n & 0xFFFFFFF0) == 0x0;
        return Modifier.isPublic(getClassModifiers(clazz)) || ((n & 0x8) != 0x0 && isSamePackage(clazz2, clazz));
    }
    
    public static boolean isTypeVisible(Class<?> componentType, Class<?> clazz) {
        if (componentType == clazz) {
            return true;
        }
        while (componentType.isArray()) {
            componentType = componentType.getComponentType();
        }
        if (componentType.isPrimitive() || componentType == Object.class) {
            return true;
        }
        ClassLoader classLoader = componentType.getClassLoader();
        final ClassLoader classLoader2 = clazz.getClassLoader();
        return classLoader == classLoader2 || ((classLoader2 != null || classLoader == null) && ((classLoader == null && componentType.getName().startsWith("java.")) || componentType == (Class)AccessController.doPrivileged(new PrivilegedAction<Class>() {
            
            @Override
            public Class<?> run() {
                try {
                    return Class.forName(name, false, classLoader2);
                }
                catch (ClassNotFoundException | LinkageError classNotFoundException | LinkageError) {
                    return null;
                }
            }
        })));
    }
    
    public static boolean isTypeVisible(MethodType methodType, Class<?> clazz) {
        int i = -1;
        while (i < methodType.parameterCount()) {
            if (!isTypeVisible((i < 0) ? methodType.returnType() : methodType.parameterType(i), clazz)) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    public static boolean isSamePackage(Class<?> clazz, Class<?> clazz2) {
        assert !clazz.isArray() && !clazz2.isArray();
        if (clazz == clazz2) {
            return true;
        }
        if (clazz.getClassLoader() != clazz2.getClassLoader()) {
            return false;
        }
        String name = clazz.getName();
        String name2 = clazz2.getName();
        int lastIndex = name.lastIndexOf(46);
        if (lastIndex != name2.lastIndexOf(46)) {
            return false;
        }
        int i = 0;
        while (i < lastIndex) {
            if (name.charAt(i) != name2.charAt(i)) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    public static String getPackageName(Class<?> clazz) {
        assert !clazz.isArray();
        String name = clazz.getName();
        int lastIndex = name.lastIndexOf(46);
        if (lastIndex < 0) {
            return "";
        }
        return name.substring(0, lastIndex);
    }
    
    public static boolean isSamePackageMember(Class<?> clazz, Class<?> clazz2) {
        return clazz == clazz2 || (isSamePackage(clazz, clazz2) && getOutermostEnclosingClass(clazz) == getOutermostEnclosingClass(clazz2));
    }
    
    private static Class<?> getOutermostEnclosingClass(Class<?> enclosingClass) {
        for (enclosingClass = (enclosingClass = enclosingClass); (enclosingClass = enclosingClass.getEnclosingClass()) != null; ) {}
        return enclosingClass;
    }
    
    private static boolean loadersAreRelated(ClassLoader parent, ClassLoader parent2, boolean b) {
        if (parent == parent2 || parent == null || (parent2 == null && !b)) {
            return true;
        }
        while (parent2 != null) {
            if (parent2 == parent) {
                return true;
            }
            parent2 = parent2.getParent();
        }
        if (b) {
            return false;
        }
        while (parent != null) {
            if (parent == parent2) {
                return true;
            }
            parent = parent.getParent();
        }
        return false;
    }
    
    public static boolean classLoaderIsAncestor(Class<?> clazz, Class<?> clazz2) {
        return loadersAreRelated(clazz.getClassLoader(), clazz2.getClassLoader(), true);
    }
}
